/** Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.artifacts.sinfo.common;

import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.function.Function;

import org.dive4elements.artifactdatabase.state.ArtifactAndFacet;
import org.dive4elements.artifactdatabase.state.Facet;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.river.artifacts.common.AbstractCalculationResult;
import org.dive4elements.river.artifacts.common.GeneralResultType;
import org.dive4elements.river.artifacts.common.ResultRow;
import org.dive4elements.river.artifacts.resources.Resources;
import org.dive4elements.river.artifacts.sinfo.flood_duration.DurationWaterlevel;
import org.dive4elements.river.artifacts.sinfo.flood_duration.FloodDurationCalculationResult;
import org.dive4elements.river.artifacts.states.DefaultState.ComputeType;
import org.dive4elements.river.exports.DiagramGenerator;
import org.dive4elements.river.themes.ThemeDocument;

import gnu.trove.TDoubleArrayList;

/**
 * Processor to generate the facet and data series of infrastructure flood durations
 *
 * @author Matthias Schäfer
 *
 */
public final class FloodDurationProcessor extends AbstractSInfoProcessor {

    private static final String FACET_FLOOD_DURATION = "sinfo_facet_flood_duration";

    private static final String FACET_FLOOD_DURATION_DESCRIPTION = "sinfo_facet_flood_duration.description";

    private static final String FACET_MAIN_VALUE_DURATION = "mainvalue.duration";

    private static final String FACET_MAIN_VALUE_DURATION_DESCRIPTION = "mainvalue.duration.description";

    private static final String I18N_AXIS_LABEL = "sinfo.chart.flood_duration.section.yaxis.label";

    private static final Set<String> HANDLED_FACET_TYPES = new HashSet<>();

    static {
        HANDLED_FACET_TYPES.add(FACET_FLOOD_DURATION);
        HANDLED_FACET_TYPES.add(FACET_MAIN_VALUE_DURATION);

    }

    public FloodDurationProcessor() {
        super(I18N_AXIS_LABEL, HANDLED_FACET_TYPES);
    }

    public static Facet createFloodDurationFacet(final CallContext context, final String hash, final String id, final AbstractCalculationResult result,
            final int resultIndex) {

        return AbstractSInfoProcessor.createFacet(context, hash, id, result, resultIndex, -1, I18N_AXIS_LABEL, FACET_FLOOD_DURATION,
                FACET_FLOOD_DURATION_DESCRIPTION);
    }

    public static Facet createMainValueDurationFacet(final CallContext context, final String hash, final String id, final FloodDurationCalculationResult result,
            final int facetIndex, final int resultIndex, final int dataIndex) {

        final String description = Resources.getMsg(context.getMeta(), FACET_MAIN_VALUE_DURATION_DESCRIPTION, FACET_MAIN_VALUE_DURATION_DESCRIPTION,
                result.getMainValueLabel(resultIndex));

        return new SInfoResultFacet(facetIndex, resultIndex, dataIndex, FACET_MAIN_VALUE_DURATION, description, I18N_AXIS_LABEL, ComputeType.ADVANCE, id, hash);
    }

    @Override
    protected String generateSeries(final DiagramGenerator generator, final ArtifactAndFacet bundle, final ThemeDocument theme, final boolean visible) {
        final String facetName = bundle.getFacetName();

        if (FACET_FLOOD_DURATION.contentEquals(facetName))
            return buildSeriesForType(generator, bundle, theme, visible, SInfoResultType.floodDuration, null);

        if (FACET_MAIN_VALUE_DURATION.contentEquals(facetName)) {

            final Function<DurationWaterlevel, Double> valueGetter = new Function<DurationWaterlevel, Double>() {
                @Override
                public Double apply(final DurationWaterlevel waterlevel) {
                    return (double) waterlevel.getFloodDurDaysPerYear();
                }
            };

            // FIXME: instead (see CollisionXXProcessor) delegate to result
            final double[][] points = getMainValueDurationPoints(generator, bundle, valueGetter);

            return buildSeriesForType(points, generator, bundle, theme, visible, null);
        }

        final String error = String.format("Unknown facet name: %s", facetName);
        throw new UnsupportedOperationException(error);
    }

    public static final double[][] getMainValueDurationPoints(final DiagramGenerator generator, final ArtifactAndFacet bundle,
            final Function<DurationWaterlevel, Double> valueGetter) {

        final AbstractCalculationResult data = getResult(generator, bundle);

        final int index = getDataIndex(bundle);

        ((SInfoResultFacet) bundle.getFacet()).getDataIndex();

        final Collection<ResultRow> rows = data.getRows();

        final TDoubleArrayList xPoints = new TDoubleArrayList(rows.size());
        final TDoubleArrayList yPoints = new TDoubleArrayList(rows.size());

        for (final ResultRow row : rows) {

            final double station = row.getDoubleValue(GeneralResultType.station);

            final List<DurationWaterlevel> waterlevels = (List<DurationWaterlevel>) row.getValue(SInfoResultType.customMultiRowColWaterlevel);
            final DurationWaterlevel waterlevel = waterlevels.get(index);

            final Double value = valueGetter.apply(waterlevel);

            xPoints.add(station);
            yPoints.add(value);
        }

        return new double[][] { xPoints.toNativeArray(), yPoints.toNativeArray() };
    }
}